//Digest access authentication
//digest.js - version 20220222"
//LNT <lnt@ly-le.info>

var http = require("http")
  , https = require("https")
  , crypto = require('crypto')
  , {URL} = require('url')

class httpDigest {
  constructor (fullUrl) {
    this.url = new URL(fullUrl)
  }
  request (callback) {
    let request_method = this.url.protocol == 'https:' ? https.request : http.request
    request_method (this.url, (res) => {
      if (res.statusCode === 401 && res.headers['www-authenticate']) {
        let wwwAuthenticate = res.headers["www-authenticate"]
        this._handle_response (wwwAuthenticate, callback)
      }
      callback(res)
    }).on("error", (e) => {
      console.error(e);
    }).end();
  }

  _handle_response (wwwAuthenticate, callback) {
    let headers = {}
    headers.Authorization = this.digestAuthHeader(wwwAuthenticate)
    let request_method = this.url.protocol == 'https:' ? https.request : http.request
    request_method (this.url, { headers: headers }, (res) => {
      callback(res)
    }).on("error", (e) => {
      console.error(e);
    }).end()
  }

  digestAuthHeader(wwwAuth) {
    let nc = 0
    wwwAuth = wwwAuth.replace(/^Digest /,'')
    let parts = wwwAuth.split(",");
    let length = parts.length;
    let params = {};
    for (var i = 0; i < length; i++) {
      let part = parts[i].match(/^\s*(\w+)=['"]([^'"]+)['"]\s*$/);
      if (part) {
        params[part[1]] = part[2];
      }
    }
    if (!params.realm || !params.nonce) return ''
    let qop = params.qop || ''
    if (qop) qop = qop.split(',')[0]
    let cnonce = crypto.randomBytes(8).toString('hex')
    let ha1 = crypto.createHash("md5").update([this.url.username, params.realm, this.url.password].join(":")).digest("hex")
    let ha2 = crypto.createHash("md5").update(['GET', this.url.pathname + this.url.search].join(":")).digest("hex")
    let response = crypto.createHash("md5").update([ha1, params.nonce, nc, cnonce, qop, ha2].join(":")).digest("hex")
    let authStr = 'Digest username="' + this.url.username + '", realm="' + params.realm
      + '", nonce="' + params.nonce + '", uri="' + this.url.pathname + this.url.search
      + '", response="' + response + '"'
    if (params.opaque) authStr += ', opaque="' + params.opaque + '"'
    if (qop) authStr +=', qop=' + qop + ', nc=' + nc + ', cnonce="' + cnonce + '"'
    return authStr
  }
}

module.exports = httpDigest
