//Record video from Onvif camera
//fRec.js - version 20220222"
//LNT <lnt@ly-le.info>

let childProcess = require('child_process')
  , fs = require('fs')
//  , digest = require('./digest')

class ffRecorder {
  constructor (options) {
    this.opts = options
    this.wStream = null
    this.timeout = null
    this.count = 0
  }

  mRecord() {
    let d = new Date()
    let d1 = d.toLocaleDateString().replaceAll('/','-')
    let path = this.opts.outFolder + '/' + d1
    if (!fs.existsSync(path)) fs.mkdirSync(path, { recursive: true })
    path += '/' + d1 + '_' + d.toLocaleTimeString()
    let clip = [
      '-hide_banner',
      '-rtsp_transport', 'tcp',
      '-i', this.opts.url,
      '-c', 'copy',
      '-f', 'segment',
      '-segment_time', this.opts.clipInterval,
      path + '_%2d.' + this.opts.outType
    ]
    this.snapshot(this.opts.imageUrl, path + '.jpg')
    return childProcess.spawn('ffmpeg', clip, {detached: false, stdio: 'ignore'})
  }

  startRecording() {
    if (!this.wStream) {
      this.wStream = this.mRecord()
      this.count = 0
      if (this.opts.logLevel != 'quiet') {
        console.log(new Date().toLocaleTimeString(), '>> Start recording...')
      }
    } else if (this.timeout) {
      clearTimeout(this.timeout)
      this.timeout = null
    }
    this.count++
  }

  stopRecording() {
    if (this.wStream) {
      if (this.count) this.count--
      if (!this.count) this.endRecording()
      else if (!this.timeout) this.timeout = setTimeout(() => { this.endRecording() }, 2100)
    } else this.count = 0
  }

  endRecording() {
    if (this.wStream) {
      this.wStream.kill()
      this.wStream = null
      if (this.timeout) clearTimeout(this.timeout)
      this.timeout = null
      this.count = 0
      if (this.opts.logLevel != 'quiet') {
        console.log(new Date().toLocaleTimeString(), '>> Stop recording!')
      }
    }
  }

  snapshot(fullUrl, filename) {
//    new digest(fullUrl).request((res) => { res.pipe(fs.createWriteStream(filename)) })
    childProcess.spawn('wget', ['-q', '-b', '-O', filename, fullUrl], {detached: false, stdio: 'ignore'})
  }
}

module.exports = ffRecorder
