//Record video from Onvif camera
//pMotionRecorder.js - version 20220222"
//LNT <lnt@ly-le.info>

const fRec = require('./fRec.js')
    , Cam  = require('onvif').Cam
    , flow = require('nimble')

class pMotionRecorder {
  constructor(options){
    let opts = options
    if (!opts.logLevel) opts.logLevel = 'normal'
    if (!opts.outType) opts.outType = 'mkv'
    if (!opts.outFolder) opts.outFolder = '/camera'
    if (isNaN(opts.clipInterval)) opts.clipInterval = 30

    this.rec = new fRec({
      outFolder: opts.outFolder,
      outType: opts.outType,
      clipInterval: opts.clipInterval,
      logLevel: opts.logLevel
    })
    let oCam = new Cam (
      {
        hostname: opts.hostname,
        username: opts.camUser,
        password: opts.camPass
      },
      (e) => {
        if (e) { console.error(e); return }
        const logLevel = opts.logLevel
        if (logLevel != 'quiet') console.log('Connected to ONVIF Device on', new Date().toLocaleDateString())
        flow.series([
          (callback) => {
            oCam.getStreamUri((e, data, xml) => {
              if (e) { console.error(e); return }
              this.rec.opts.url = (data.uri).replace(oCam.hostname, oCam.username + ':' + oCam.password + '@' + oCam.hostname)
            })
            callback()
          },
          (callback) => {
            oCam.getSnapshotUri((e, data, xml) => {
              if (e) { console.error(e); return }
              this.rec.opts.imageUrl = (data.uri).replace(oCam.hostname, oCam.username + ':' + oCam.password + '@' + oCam.hostname)
            })
            callback()
          },
          (callback) => {
            oCam.on('event', (camMsg, xml) => {
              let eTopic = camMsg.topic._.split(':').pop()
              if (eTopic == 'VideoSource/MotionAlarm') {
                let eOperation = camMsg.message.message.$.PropertyOperation,
                    eValue = camMsg.message.message.data.simpleItem.$.Value
                this.processEvent(eOperation, eValue, logLevel)
              }
            })
            callback()
          }
        ])
      }
    )
  }

  processEvent (operation, value, logLevel) {
    if (logLevel == 'debug')
      console.log(new Date().toLocaleTimeString(), 'Status:', operation, ', Motion:', value)
    if (operation == 'Changed') {
      if (value) this.rec.startRecording(); else this.rec.stopRecording()
    }
  }
}

module.exports = pMotionRecorder
