//Record video from Onvif camera
//sMotionRecorder.js - version 20220217"
//LNT <lnt@ly-le.info>

const fRec = require('./fRec.js')
    , Cam  = require('onvif').Cam
    , flow = require('nimble')
    , http = require('http')
    , ip = require('ip')

class sMotionRecorder {
  constructor (options) {
    let opts = options
    if (!opts.logLevel) opts.logLevel = 'normal'
    if (!opts.outType) opts.outType = 'mkv'
    if (!opts.outFolder) opts.outFolder = '/camera'
    if (isNaN(opts.clipInterval)) opts.clipInterval = 45
    const serverPort = 8899
    const logLevel = opts.logLevel
    this.rec = new fRec ({
      outFolder: opts.outFolder,
      outType: opts.outType,
      clipInterval: opts.clipInterval,
      logLevel: logLevel
    })
    let oCam = new Cam ({
      hostname: opts.hostname,
      username: opts.camUser,
      password: opts.camPass
    },
    (e) => {
      if (e) { console.error(e); return; }
      if (logLevel != 'quiet') console.log('Connected to ONVIF Device on', new Date().toLocaleDateString())
      flow.series([
        (callback) => {
          oCam.getStreamUri((e, data, xml) => {
            if (e) { console.error(e); return }
            this.rec.opts.url = (data.uri).replace(oCam.hostname, oCam.username + ':' + oCam.password + '@' + oCam.hostname)
            callback()
          })
        },
        (callback) => {
          oCam.getSnapshotUri((e, data, xml) => {
            if (e) { console.error(e); return }
            this.rec.opts.imageUrl = (data.uri).replace(oCam.hostname, oCam.username + ':' + oCam.password + '@' + oCam.hostname)
            callback()
          })
        },
        (callback) => {
          let camId = Math.random().toString(36).substring(2, 10)
          let receiveUrl = 'http://' + ip.address() + ':' + serverPort + '/events/' + camId
          oCam.subscribe (
            {url: receiveUrl},
            (e, subscription, xml) => {
               if (e) { console.error(e); return }
            }
          )
          callback()
        }
      ])
    })
    http.createServer((req, res) => {
      let body = ''
      req.on('data', chunk => { body += chunk })
      req.on('end', () => {
        if (req.method == "POST" && /VideoSource\/MotionAlarm/.test(body)) {
console.log(req)
          let eOperation = body.match(/PropertyOperation="([^"]+)/)[1],
              eValue = body.match(/Name="State" Value="([^"]+)/)[1]
          this.processEvent(eOperation, eValue, logLevel)
        }
        res.writeHead(200, { "Content-Type": "text/plain" });
      })
    }).listen(serverPort)
  }

  processEvent (operation, value, logLevel) {
    if (logLevel == 'debug')
      console.log(new Date().toLocaleTimeString(), 'Status:', operation, ', Motion:', value)
    if (operation == 'Changed') {
      if (value === 'true') this.rec.startRecording(); else this.rec.stopRecording()
    }
  }
}

module.exports = sMotionRecorder
